dojo.provide("internal.composites.PictureUploader");

// Need these for things referenced in here
dojo.require("dojox.form.FileUploader");
dojo.require("dijit._Widget");
// FIXME: Look at switching to dojox.???._Templated
// (This would let me decide whether to show the license combo or not)
dojo.require("dijit._Templated");

// These are referenced in the template
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboBox");

dojo.declare("internal.composites.PictureUploader",
	[dijit._Widget, dijit._Templated],
	{
		//  summary: A composite picture uploader and license selector
		//
		// description: A control that lets you choose an image file, associate a
		// license, and then upload them. You'll have to deal with the license
		// yourself

		// The ID of the transaction associated with this upload
		// This is something that you should override
		transaction_id: "some_guid",

		// upload_url: String (actually, a URL)
		// Where to post to when uploading
		// Remember, there are issues with the file upload control and
		// subdirectories
		upload_url: "you must specify this",

		// browse_label: String
		// The title of the "Browse for file" button
		browse_label: "Browse...",

			// fileMask: Array of string pairs
			// Only allow uploading certain file types
			// Each pair is an array.  Member 0 describes the file type.  Member 1 is
			// the file mask to apply to that type
			fileMask: [
				["Jpeg File",	"*.jpg;*.jpeg"],
				["GIF File",	"*.gif"],
				["PNG File",	"*.png"],
				["All Images",	"*.jpg;*.jpeg;*.gif;*.png"]
			],

			// selectMultipleFiles: Boolean
			// Allow the user to select more than one file to upload?
			selectMultipleFiles: false,

		/////////////////////////////////////////////////////////////
		// Probably no reason to change any of these
		/////////////////////////////////////////////////////////////

		// id: String
		// The id for the div which wraps this control
		// FIXME: Do I care about this?
		id: "pictureUploader", 

		// file_name: HTML for showing which file's going to be uploaded
		// ID of whichever element to use to indicate the name of the file 
		file_name: null,

		// upload_progress: HTML element for reporting upload progress
		// ID of the element that tracks upload progress
		// FIXME: this should probably be a status bar
		upload_progress: null,

		// upload_results: HTML element that can report upload results
		// ID of the element that shows the results
		upload_results: null,

		// uploadButton: dijit.form.Button (actually, pretty much anything)
		// Member variable that points to the button the FileUploader dijit
		// replaces
		uploadButton: null,

		// uploader: dojox.form.FileUploader
		// Member variable that points to the FileUploader dijit
		uploader: null,

		// widgetsInTemplate: Boolean
		// Make widget expansion happen (a _Templated member variable)
		widgetsInTemplate: true,

		// templatePath: String
		// how do we find the template file for this widget?
		templatePath: dojo.moduleUrl("internal.composites",
		"PictureUploader/PictureUploader.html"),

		// nonRequired: String
		// This way, the build system knows about me
		// FIXME: What is this actually for?
		nonRequired: "",

		// Initialization function to run on startup
		postCreate: function(){
			// assign a file uploader to the appropriate button
			this.uploader = new dojox.form.FileUploader({
				button: this.uploadButton,
				degrabable : true,
				uploadUrl: "upload?id=" + this.transaction_id,
				uploadOnChange: false,
				selectMultipleFiles: this.selectMultipleFiles,
				fileMask: this.fileMask,
				isDebug:true
			});

			this.connect(this.uploader, "onProgress", "_onProgress");
			this.connect(this.uploader, "onChange", "_onChange");
			this.connect(this.uploader, "onComplete", "_onComplete");
		},

		upload: function(){
			this.uploader.upload();
		},

		/////////////////////////////////////////////////
		// Event handlers for file upload progress
		/////////////////////////////////////////////////

		// select a file
		_onChange: function(dataArray){
			dojo.forEach(dataArray, function(data){
				if(this.selectMultipleFiles){
					this.file_name.innerHTML += "File to upload: " + 
						data.name+" " + Math.ceil(data.size*.001)+"kb \n";
				}else{
					var msg = "File to upload: " + data.name
						+ " " + Math.ceil(data.size*.001)+"kb \n";

					this.file_name.innerHTML = msg;
				}
			}, this);
		},

		// upload progress
		_onProgress: function(fileList){
			// Note that this doesn't get fired unless the server is configured to
			// report progress
			this.upload_progress.innerHTML = "";

			// The forEach is for handling multiple files
			dojo.forEach(fileList, function(fileInfo){
				var progress = "(" + fileInfo.percent + "%) " + fileInfo.name;
				this.upload_progress.innerHTML += progress;

				// Kludge because onComplete isn&#8217;t getting called:
				if(fileInfo.bytesLoaded == fileInfo.bytesTotal){
					// Do another AJAX postback to get the URL to the image
					dojo.xhrGet({
						// Controller to get the URL of the picture that's associated
						// with this transaction
						url: "/beta/test/get_url?id=" + this.transaction_id,
						handleAs: "text",
						handle: dojo.hitch(this, function(url, args){
							if(typeof url == "error"){
								alert("Error!");
								console.log(args);
							}else{
								console.log("URL: " + url);
								this.result.innerHTML = '<img src="' + url + '" />';
							}
						})
					});
				}
			}, this);
		},

		// And mark the upload finished:
		_onComplete: function(dataArray){
			console.log("Upload complete");
			console.dir(dataArray);
			this.upload_results.innerHTML = "";
			this.upload_progress.innerHTML = "";
			dojo.forEach(dataArray, function(data){
				this.upload_progress.innerHTML += data.name + "<br />";
				this.upload_results.innerHTML += '<img src="' + data + '" />';
				console.dir(data);
			}, this);
		}
	});
